# Honeybee: A Plugin for Environmental Analysis (GPL)
# This file is part of Honeybee.
#
# Copyright (c) 2022, Ladybug Tools.
# You should have received a copy of the GNU Affero General Public License
# along with Honeybee; If not, see <http://www.gnu.org/licenses/>.
# 
# @license AGPL-3.0-or-later <https://spdx.org/licenses/AGPL-3.0-or-later>

"""
Create a uniform sky that yields a certain illuminance.

-
    Args:
        _value_: Desired value for sky horizontal illuminance in lux. (Default: 10000).

    Returns:
        sky: A honeybee sky that can be used to create a point-in-time recipe.
"""

ghenv.Component.Name = 'HB Certain Illuminance'
ghenv.Component.NickName = 'CertainIllum'
ghenv.Component.Message = '1.4.0'
ghenv.Component.Category = 'HB-Radiance'
ghenv.Component.SubCategory = '2 :: Light Sources'
ghenv.Component.AdditionalHelpFromDocStrings = '1'

try:
    from honeybee_radiance.lightsource.sky import CertainIrradiance
except ImportError as e:
    raise ImportError('\nFailed to import honeybee_radiance:\n\t{}'.format(e))


_value_ = 10000 if _value_ is None else _value_
sky = CertainIrradiance.from_illuminance(_value_)



# Honeybee: A Plugin for Environmental Analysis (GPL)
# This file is part of Honeybee.
#
# Copyright (c) 2022, Ladybug Tools.
# You should have received a copy of the GNU Affero General Public License
# along with Honeybee; If not, see <http://www.gnu.org/licenses/>.
# 
# @license AGPL-3.0-or-later <https://spdx.org/licenses/AGPL-3.0-or-later>

"""
Create a WEA object using the original ASHRAE Clear Sky formula.
-

    Args:
        _location: A Ladybug Location object which will set the sun poisition for
            the clear sky Wea. Locations can be obtained from the "LB Import
            EPW" or the "LB Construct Location" component.
        clearness_: A factor to be multiplied by the output of the clear sky model.
            This is to help account for locations where clear, dry skies predominate
            (e.g., at high elevations) or, conversely, where hazy and humid conditions
            are frequent. See Threlkeld and Jordan (1958) for recommended values.
            Typical values range from 0.95 to 1.05 and are usually never more
            than 1.2. (Default: 1.0).
        hoys_: An optional list of hours of the year (numbers from 0 to 8759) for
            which the Wea will be filtered. HOYs can be generated from the
            "LB Analysis Period" component or they can be obtained through
            other means like analysis of the values in an occupancy schedule.
            By default, the Wea will be generated for the whole year.
        timestep_: An integer representing the timestep with which to make the
            WEA object. (Default: 1, for 1 step per hour of the year).

    Returns:
        wea: A wea object from stat file. This wea object represents an original 
            ASHRAE Clear Sky, which is intended to determine peak solar load and
            sizing parmeters for HVAC systems.
"""

ghenv.Component.Name = 'HB Wea From Clear Sky'
ghenv.Component.NickName = 'ClearSky'
ghenv.Component.Message = '1.4.0'
ghenv.Component.Category = 'HB-Radiance'
ghenv.Component.SubCategory = '2 :: Light Sources'
ghenv.Component.AdditionalHelpFromDocStrings = '2'

try:
    from ladybug.wea import Wea
except ImportError as e:
    raise ImportError('\nFailed to import honeybee:\n\t{}'.format(e))

try:  # import ladybug_rhino dependencies
    from ladybug_rhino.grasshopper import all_required_inputs
except ImportError as e:
    raise ImportError('\nFailed to import ladybug_rhino:\n\t{}'.format(e))


if all_required_inputs(ghenv.Component):
    timestep_ = 1 if timestep_ is None else timestep_
    clearness_ = 1 if clearness_ is None else clearness_
    wea = Wea.from_ashrae_clear_sky(_location, clearness_, timestep_)
    if len(hoys_) != 0:
        wea = wea.filter_by_hoys(hoys_)


# Honeybee: A Plugin for Environmental Analysis (GPL)
# This file is part of Honeybee.
#
# Copyright (c) 2022, Ladybug Tools.
# You should have received a copy of the GNU Affero General Public License
# along with Honeybee; If not, see <http://www.gnu.org/licenses/>.
# 
# @license AGPL-3.0-or-later <https://spdx.org/licenses/AGPL-3.0-or-later>

"""
Create a Honeybee View for an image-based analysis.
-

    Args:
        _name_: Text to set the name for the modifier and to be incorporated into
            a unique modifier identifier.
        _position: An point to set the position of the view in 3D space (-vp).
            This is the focal point of a perspective view or the center of a
            parallel projection.
        _direction: A vector for the direction that the veiw is facing (-vd).
            The length of this vector indicates the focal distance as needed by
            the pixel depth of field (-pd) in rpict.
        _up_vector_: An optional vector to set the vertical direction of the
            view (-vu). (Default: (0, 0, 1)).
        _view_type_: An integer to set the view type (-vt). Choose from the choices
            below. (Default: 0).
                * 0 Perspective (v)
                * 1 Hemispherical fisheye (h)
                * 2 Parallel (l)
                * 3 Cylindrical panorama (c)
                * 4 Angular fisheye (a)
                * 5 Planisphere [stereographic] projection (s)
            For more detailed description about view types check rpict manual
            page (http://radsite.lbl.gov/radiance/man_html/rpict.1.html)
        _h_angle_: A number for the view horizontal size (-vh) in degrees. For a
            perspective projection (including fisheye views), val is the horizontal
            field of view. For a parallel projection, val is the view width in
            world coordinates. (Default: 60).
        _v_angle_: A number for the view vertical size (-vv) in degrees. For
            a perspective projection (including fisheye views), val is the horizontal
            field of view. For a parallel projection, val is the view width in
            world coordinates. (Default: 60).

    Returns:
        view: A Honeybee View object that can be used in an image-based recipe.
"""

ghenv.Component.Name = 'HB View'
ghenv.Component.NickName = 'View'
ghenv.Component.Message = '1.4.1'
ghenv.Component.Category = 'HB-Radiance'
ghenv.Component.SubCategory = '0 :: Basic Properties'
ghenv.Component.AdditionalHelpFromDocStrings = '4'

try:  # import the core honeybee dependencies
    from honeybee.typing import clean_and_id_rad_string, clean_rad_string
except ImportError as e:
    raise ImportError('\nFailed to import honeybee:\n\t{}'.format(e))

try:
    from honeybee_radiance.view import View
except ImportError as e:
    raise ImportError('\nFailed to import honeybee:\n\t{}'.format(e))

try:  # import ladybug_rhino dependencies
    from ladybug_rhino.grasshopper import all_required_inputs
except ImportError as e:
    raise ImportError('\nFailed to import ladybug_rhino:\n\t{}'.format(e))

VIEW_TYPES = ('v', 'h', 'l', 'c', 'a', 's')


if all_required_inputs(ghenv.Component):
    # process the points/vectors into tuples
    _pos = (_position.X, _position.Y, _position.Z)
    _dir = (_direction.X, _direction.Y, _direction.Z)

    # set the default values
    name = clean_and_id_rad_string('View') if _name_ is None else _name_
    _up_vec = (_up_vector_.X, _up_vector_.Y, _up_vector_.Z) if _up_vector_ \
        is not None else (0, 0, 1)
    if _view_type_ is None:
        _type_= 'v'
    else:
        _type_= _view_type_ if _view_type_ in VIEW_TYPES else VIEW_TYPES[int(_view_type_)]
    _h_angle_ = 60 if _h_angle_ is None else _h_angle_
    _v_angle_ = 60 if _v_angle_ is None else _v_angle_

    view = View(clean_rad_string(name), _pos, _dir, _up_vec, _type_,
                _h_angle_, _v_angle_)
    view.standardize_fisheye()
    if _name_ is not None:
        view.display_name = _name_



# Honeybee: A Plugin for Environmental Analysis (GPL)
# This file is part of Honeybee.
#
# Copyright (c) 2022, Ladybug Tools.
# You should have received a copy of the GNU Affero General Public License
# along with Honeybee; If not, see <http://www.gnu.org/licenses/>.
# 
# @license AGPL-3.0-or-later <https://spdx.org/licenses/AGPL-3.0-or-later>

"""
Create a Sensor Grid object that can be used in a grid-based recipe.
-

    Args:
        _name_: A name for this sensor grid.
        _positions: A list or a datatree of points with one point for the position
            of eah sensor. Each branch of the datatree will be considered as a
            separate sensor grid.
        _directions_: A list or a datatree of vectors with one vector for the
            direction of each sensor. The input here MUST therefor align with
            the input _positions. If no value is provided (0, 0, 1) will be
            assigned for all the sensors.
        mesh_: An optional mesh that aligns with the sensors. This is useful for
            generating visualizations of the sensor grid beyond the sensor
            positions. Note that the number of sensors in the grid must match
            the number of faces or the number vertices within the mesh.
        base_geo_: An optional Brep for the geometry used to make the grid. There are
            no restrictions on how this brep relates to the sensors and it is
            provided only to assist with the display of the grid when the number
            of sensors or the mesh is too large to be practically visualized.

    Returns:
        grid: An SensorGrid object that can be used in a grid-based recipe.
"""

ghenv.Component.Name = 'HB Sensor Grid'
ghenv.Component.NickName = 'SensorGrid'
ghenv.Component.Message = '1.4.0'
ghenv.Component.Category = 'HB-Radiance'
ghenv.Component.SubCategory = '0 :: Basic Properties'
ghenv.Component.AdditionalHelpFromDocStrings = '4'

try:  # import the core honeybee dependencies
    from honeybee.typing import clean_and_id_rad_string, clean_rad_string
except ImportError as e:
    raise ImportError('\nFailed to import honeybee:\n\t{}'.format(e))

try:  # import the honeybee-radiance dependencies
    from honeybee_radiance.sensorgrid import SensorGrid
except ImportError as e:
    raise ImportError('\nFailed to import honeybee_radiance:\n\t{}'.format(e))

try:  # import ladybug_rhino dependencies
    from ladybug_rhino.grasshopper import all_required_inputs
    from ladybug_rhino.togeometry import to_mesh3d, to_face3d
except ImportError as e:
    raise ImportError('\nFailed to import ladybug_rhino:\n\t{}'.format(e))


if all_required_inputs(ghenv.Component):
    # set the default name and process the points to tuples
    name = clean_and_id_rad_string('SensorGrid') if _name_ is None else _name_
    pts = [(pt.X, pt.Y, pt.Z) for pt in _positions]

    # create the sensor grid object
    id  = clean_rad_string(name) if '/' not in name else clean_rad_string(name.split('/')[0])
    if len(_directions_) == 0:
        grid = SensorGrid.from_planar_positions(id, pts, (0, 0, 1))
    else:
        vecs = [(vec.X, vec.Y, vec.Z) for vec in _directions_]
        grid = SensorGrid.from_position_and_direction(id, pts, vecs)

    # set the display name
    if _name_ is not None:
        grid.display_name = _name_
    if '/' in name:
        grid.group_identifier = \
            '/'.join(clean_rad_string(key) for key in name.split('/')[1:])
    if mesh_ is not None:
        grid.mesh = to_mesh3d(mesh_)
    if base_geo_ is not None:
        grid.base_geometry = to_face3d(base_geo_)



# Honeybee: A Plugin for Environmental Analysis (GPL)
# This file is part of Honeybee.
#
# Copyright (c) 2022, Ladybug Tools.
# You should have received a copy of the GNU Affero General Public License
# along with Honeybee; If not, see <http://www.gnu.org/licenses/>.
# 
# @license AGPL-3.0-or-later <https://spdx.org/licenses/AGPL-3.0-or-later>

"""
Construct a WEA from hourly data collections and the Zhang-Huang Solar Model.
-

    Args:
        _location: A Ladybug Location object.
        _cloud_cover: Hourly DataCollection with the fraction of total sky cover
            (tenths of coverage). (eg. 1 is 1/10 covered. 10 is total coverage)
        _rel_humidity: Hourly DataCollection with relative humidity [%].
        _dry_bulb_temp: Hourly DataCollection with dry bulb temperature [C].
        _wind_speed: Hourly DataCollection with wind speed [m/s].
        _atmos_pressure_: Hourly DataCollection with amtospheric pressure [Pa].
            If no value is connected here, pressure at sea level will be
            assumed (101,325 Pa).

    Returns:
        wea: A wea object from the input data collections and the Zhang-Huang
            solar model.
"""

ghenv.Component.Name = 'HB Wea from Zhang-Huang'
ghenv.Component.NickName = 'Zhang-Huang'
ghenv.Component.Message = '1.4.0'
ghenv.Component.Category = 'HB-Radiance'
ghenv.Component.SubCategory = '2 :: Light Sources'
ghenv.Component.AdditionalHelpFromDocStrings = '2'

try:
    from ladybug.wea import Wea
    from ladybug.datacollection import HourlyContinuousCollection
    from ladybug.header import Header
    from ladybug.datatype.pressure import AtmosphericStationPressure
except ImportError as e:
    raise ImportError('\nFailed to import ladybug:\n\t{}'.format(e))

try:
    from ladybug_rhino.grasshopper import all_required_inputs
except ImportError as e:
    raise ImportError('\nFailed to import ladybug_rhino:\n\t{}'.format(e))


if all_required_inputs(ghenv.Component):
    # perform checks.
    assert isinstance(_cloud_cover, HourlyContinuousCollection), \
        'Data Collections must be Continuous Hourly.'
    if _atmos_pressure_ is None:
        header = Header(AtmosphericStationPressure(), 'Pa',
            _cloud_cover.header.analysis_period, _cloud_cover.header.metadata)
        _atmos_pressure_ = HourlyContinuousCollection(header,
            [101325] * 8760 * _cloud_cover.header.analysis_period.timestep)

    # build the Wea
    wea = Wea.from_zhang_huang_solar(_location, _cloud_cover, _rel_humidity,
        _dry_bulb_temp, _wind_speed, _atmos_pressure_)


# This file is part of Honeybee.
#
# Copyright (c) 2022, Ladybug Tools.
# You should have received a copy of the GNU Affero General Public License
# along with Honeybee; If not, see <http://www.gnu.org/licenses/>.
# 
# @license AGPL-3.0-or-later <https://spdx.org/licenses/AGPL-3.0-or-later>

"""
Import the hourly illuminance or irradiance results of an annual daylight or irradiance
study to a list of data collections.
_
The resulting data collections can be visulized using the ladybug components or
deconstructed for detailed analysis with native Grasshopper math components.

-
    Args:
        _results: An list of annual Radiance result files from the "HB Annual Daylight"
            component (containing the .ill files and the sun-up-hours.txt).
            This can also be just the path to the folder containing these
            result files.
        sel_pts_: An optional point or list of points, which will be used to filter
            the sensors for which data collections will be imported. If there
            is an input here, the all_pts_ must be connected.
        all_pts_: The data tree of all sensor points that were used in the simulation.
            This is required in order to look up the index of the sel_pts_ in
            the results matrices. 

 Returns:
        report: Reports, errors, warnings, etc.
        data: A list of hourly data collections containing illuminance or irradiance results.
            These can be visulized using the ladybug components or deconstructed
            for detailed analysis with native Grasshopper math components.
"""

ghenv.Component.Name = 'HB Annual Results to Data'
ghenv.Component.NickName = 'AnnualToData'
ghenv.Component.Message = '1.4.0'
ghenv.Component.Category = 'HB-Radiance'
ghenv.Component.SubCategory = '4 :: Results'
ghenv.Component.AdditionalHelpFromDocStrings = '2'

import os

try:
    from ladybug.datatype.illuminance import Illuminance
    from ladybug.datatype.energyflux import Irradiance
    from ladybug.datatype.time import Time
    from ladybug.analysisperiod import AnalysisPeriod
    from ladybug.header import Header
    from ladybug.datacollection import HourlyContinuousCollection
except ImportError as e:
    raise ImportError('\nFailed to import ladybug:\n\t{}'.format(e))

try:
    from honeybee_radiance.postprocess.annualdaylight import _process_input_folder
except ImportError as e:
    raise ImportError('\nFailed to import honeybee_radiance:\n\t{}'.format(e))

try:
    from ladybug_rhino.config import tolerance
    from ladybug_rhino.togeometry import to_point3d
    from ladybug_rhino.grasshopper import all_required_inputs, list_to_data_tree, \
        data_tree_to_list
except ImportError as e:
    raise ImportError('\nFailed to import ladybug_rhino:\n\t{}'.format(e))


def file_to_data(ill_file, point_filter, su_pattern, header, timestep):
    """Get a list of data collections for a given result file."""
    data_colls = []
    with open(ill_file) as results:
        if point_filter is None:
            for pt_res in results:
                base_values = [0] * 8760 * timestep
                for val, hr in zip(pt_res.split(), su_pattern):
                    base_values[hr] = float(val)
                data_colls.append(HourlyContinuousCollection(header, base_values))
        else:
            for i, pt_res in enumerate(results):
                if i in point_filter:
                    base_values = [0] * 8760 * timestep
                    for val, hr in zip(pt_res.split(), su_pattern):
                        base_values[hr] = float(val)
                    data_colls.append(HourlyContinuousCollection(header, base_values))
    return data_colls


def find_point_in_grid(s_pt, all_pts):
    """Find the index of a point in a list of list of grids."""
    for i, grid_pts in enumerate(all_pts):
        for j, g_pt in enumerate(grid_pts):
            if g_pt.is_equivalent(s_pt, tolerance):
                return i, j


if all_required_inputs(ghenv.Component):
    # get the relevant .ill files
    res_folder = os.path.dirname(_results[0]) if os.path.isfile(_results[0]) \
        else _results[0]
    grids, sun_up_hours = _process_input_folder(res_folder, '*')

    # check the sel_pts and all_pts input
    pt_filter = [None for i in grids]
    if len(sel_pts_) != 0:
        pt_filter = [[] for i in grids]
        all_pts = [[to_point3d(pt) for pt in dat[-1]] for dat in data_tree_to_list(all_pts_)]
        assert len(all_pts) != 0, 'all_pts_ must be connected in order to use sel_pts_.'
        sel_pts = [to_point3d(pt) for pt in sel_pts_]
        for s_pt in sel_pts:
            i, j = find_point_in_grid(s_pt, all_pts)
            pt_filter[i].append(j)

    # extract the timestep if it exists
    timestep, has_t_step = 1, False
    tstep_file = os.path.join(res_folder, 'timestep.txt')
    if os.path.isfile(tstep_file):  # it's an annual irradiance simulation
        with open(tstep_file) as tf:
            timestep = int(tf.readline())
        has_t_step = True

    # parse the sun-up-hours
    sun_up_hours = [int(h * timestep) for h in sun_up_hours]

    # create the header that will be used for all of the data collections
    aper = AnalysisPeriod(timestep=timestep)
    if 'direct_sun_hours' in res_folder:
        head = Header(Time(), 'hr', aper)
    elif has_t_step:
        head = Header(Irradiance(), 'W/m2', aper)
    else:
        head = Header(Illuminance(), 'lux', aper)

    # create the data collections from the .ill files
    data = []
    for grid_info, p_filt in zip(grids, pt_filter):
        ill_file = os.path.join(res_folder, '%s.ill' % grid_info['full_id'])
        data_list = file_to_data(ill_file, p_filt, sun_up_hours, head, timestep)
        data.append(data_list)
    data = list_to_data_tree(data)



# Honeybee: A Plugin for Environmental Analysis (GPL)
# This file is part of Honeybee.
#
# Copyright (c) 2022, Ladybug Tools.
# You should have received a copy of the GNU Affero General Public License
# along with Honeybee; If not, see <http://www.gnu.org/licenses/>.
# 
# @license AGPL-3.0-or-later <https://spdx.org/licenses/AGPL-3.0-or-later>

"""
Get recommended Radiance parameters given a recipe type and a level of detail.
_
The original recommendation for the various Radiance paramters comes from this document.
http://radsite.lbl.gov/radiance/refer/Notes/rpict_options.html
_
This presentation by John Mardaljevic gives a good overview of the meaning of each
radiance paramter.
http://radiance-online.org/community/workshops/2011-berkeley-ca/presentations/day1/JM_AmbientCalculation.pdf

-
    Args:
        _recipe_type: An integer or text for the type of recipe. Acceptable text inputs are
            either the full text of the recipe type (eg. point-in-time-grid) or
            the name of the Radiance command for which the parameters are being
            used (eg. rtrace). Choose from the following options.
                * 0 | rtrace     | point-in-time-grid | daylight-factor
                * 1 | rpict      | point-in-time-view
                * 2 | rfluxmtx   | annual
        _detail_level_: An integer or text for the level of detail/quality for which
            radiance parameters will be output. (Default: 0 for low).
            Choose from the following options.
                * 0 | low
                * 1 | medium
                * 2 | high
        additional_par_: Text to override the Radiance parameters as needed. Radiance's
            standard syntax must be followed (e.g. -ps 1 -lw 0.01).

    Returns:
        rad_par: Radiance parameters as a text string. These can be plugged into the
            radiance_par_ input of the various recipes.
"""

ghenv.Component.Name = 'HB Radiance Parameter'
ghenv.Component.NickName = 'RadPar'
ghenv.Component.Message = '1.4.0'
ghenv.Component.Category = 'HB-Radiance'
ghenv.Component.SubCategory = '3 :: Recipes'
ghenv.Component.AdditionalHelpFromDocStrings = '6'

try:  # import honeybee_radiance_command dependencies
    from honeybee_radiance_command.options.rtrace import RtraceOptions
    from honeybee_radiance_command.options.rpict import RpictOptions
    from honeybee_radiance_command.options.rfluxmtx import RfluxmtxOptions
except ImportError as e:
    raise ImportError('\nFailed to import honeybee_radiance_command:\n\t{}'.format(e))

try:
    from ladybug_rhino.grasshopper import all_required_inputs, recipe_result
except ImportError as e:
    raise ImportError('\nFailed to import ladybug_rhino:\n\t{}'.format(e))

# dictionaries of the various recommendations for radiance parameters
RTRACE = {
    'ab': [2, 3, 6],
    'ad': [512, 2048, 4096],
    'as_': [128, 2048, 4096],
    'ar': [16, 64, 128],
    'aa': [.25, .2, .1],
    'dj': [0, .5, 1],
    'ds': [.5, .25, .05],
    'dt': [.5, .25, .15],
    'dc': [.25, .5, .75],
    'dr': [0, 1, 3],
    'dp': [64, 256, 512],
    'st': [.85, .5, .15],
    'lr': [4, 6, 8],
    'lw': [.05, .01, .005],
    'ss': [0, .7, 1]
}

RPICT = {
    'ab': [2, 3, 6],
    'ad': [512, 2048, 4096],
    'as_': [128, 2048, 4096],
    'ar': [16, 64, 128],
    'aa': [.25, .2, .1],
    'ps': [8, 4, 2],
    'pt': [.15, .10, .05],
    'pj': [.6, .9, .9],
    'dj': [0, .5, 1],
    'ds': [.5, .25, .05],
    'dt': [.5, .25, .15],
    'dc': [.25, .5, .75],
    'dr': [0, 1, 3],
    'dp': [64, 256, 512],
    'st': [.85, .5, .15],
    'lr': [4, 6, 8],
    'lw': [.05, .01, .005],
    'ss': [0, .7, 1]
}

RFLUXMTX = {
    'ab': [3, 5, 6],
    'ad': [5000, 15000, 25000],
    'as_': [128, 2048, 4096],
    'ds': [.5, .25, .05],
    'dt': [.5, .25, .15],
    'dc': [.25, .5, .75],
    'dr': [0, 1, 3],
    'dp': [64, 256, 512],
    'st': [.85, .5, .15],
    'lr': [4, 6, 8],
    'lw': [0.000002, 6.67E-07, 4E-07],
    'ss': [0, .7, 1],
    'c': [1, 1, 1]
}


# dictionaries to convert between input formats
RECIPE_TYPES = {
    '0': 'rtrace',
    'point-in-time-grid': 'rtrace',
    'daylight-factor': 'rtrace',
    'rtrace': 'rtrace',
    '1': 'rpict',
    'point-in-time-image': 'rpict',
    'rpict': 'rpict',
    '2': 'rfluxmtx',
    'annual': 'rfluxmtx',
    'rfluxmtx': 'rfluxmtx',
}

DETAIL_LEVELS = {
    '0': 0,
    'low': 0,
    '1': 1,
    'medium': 1,
    '2': 2,
    'high': 2
}


if all_required_inputs(ghenv.Component):
    # process the recipe type and level of detail
    _detail_level_ = DETAIL_LEVELS[_detail_level_.lower()] \
        if _detail_level_ is not None else 0
    command_name = RECIPE_TYPES[_recipe_type.lower()]
    if command_name == 'rtrace':
        option_dict = RTRACE
        option_obj = RtraceOptions()
    elif command_name == 'rpict':
        option_dict = RPICT
        option_obj = RpictOptions()
    elif command_name == 'rfluxmtx':
        option_dict = RFLUXMTX
        option_obj = RfluxmtxOptions()

    # assign the defualts to the object and output the string
    for opt_name, opt_val in option_dict.items():
        setattr(option_obj, opt_name, opt_val[_detail_level_])
    if additional_par_:
        option_obj.update_from_string(additional_par_)
    rad_par = option_obj.to_radiance()
